{*************************************************************************
                Copyright (c) PilotLogic Software House

  Package pl_ExControls
  This file is part of CodeTyphon Studio (https://www.pilotlogic.com/)

 ***** BEGIN LICENSE BLOCK *****
 * Version: LGPL-2.1 with link exception (Modified LGPL)
 *
 * The contents of this file are subject to the
 * GNU LIBRARY GENERAL PUBLIC LICENSE Version 2.1 (the "License")
 * with the following modification:
 *
 * As a special exception, the copyright holders of this library give you
 * permission to link this library with independent modules to produce an
 * executable, regardless of the license terms of these independent modules,
 * and to copy and distribute the resulting executable under terms of your choice,
 * provided that you also meet, for each linked independent module, the terms
 * and conditions of the license of that module. An independent module is a
 * module which is not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library, but
 * you are not obligated to do so. If you do not wish to do so, delete this
 * exception statement from your version.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * ***** END LICENSE BLOCK *****

***************************************************************************}

unit TplShapeProgressUnit;

{$mode objfpc}{$H+}

interface

uses
  LResources, Classes, FPCanvas, SysUtils, Controls, Graphics, Math;

type
  TplShapeType = (stText, stPie, stImage,
    stBarHor, stBarVer,
    stRoundBarVer, stRoundBarHor);


  TplShapeProgress = class(TGraphicControl)
  private
    FShapeType: TplShapeType;
    FBackColor: TColor;
    FBorderStyle: TBorderStyle;
    FCurValue: longint;
    FFillColor: TColor;
    FFillStyle: TFPBrushStyle;
    FFillImage: TPicture;
    FMaxValue: longint;
    FMinValue: longint;
    FShowText: boolean;
    function GetPercentDone: longint;
    procedure SetBackColor(AValue: TColor);
    procedure SetFillColor(AValue: TColor);
    procedure SetFillStyle(AValue: TFPBrushStyle);
    procedure SetFillImage(AValue: TPicture);
    procedure SetShapeType(const AValue: TplShapeType);
    procedure SetMaxValue(AValue: longint);
    procedure SetMinValue(AValue: longint);
    procedure SetProgress(AValue: longint);
    procedure SetShowText(AValue: boolean);
    procedure DrawRoundBar;
    procedure DrawBar;
    procedure DrawText;
    procedure DrawPie;
    procedure DrawImage;
  public
    constructor Create(TheOwner: TComponent); override;
    destructor Destroy; override;
    procedure Paint; override;
    procedure AddProgress(AValue: longint);
    property PercentDone: longint read GetPercentDone;
  published
    property BackColor: TColor read FBackColor write SetBackColor default clWhite;
    property FillColor: TColor read FFillColor write SetFillColor default clBlack;
    property FillStyle: TFPBrushStyle read FFillStyle write SetFillStyle default bsSolid;
    property FillImage: TPicture read FFillImage write SetFillImage;
    property ShapeType: TplShapeType read FShapeType write SetShapeType default stPie;
    property MinValue: longint read FMinValue write SetMinValue default 0;
    property MaxValue: longint read FMaxValue write SetMaxValue default 100;
    property Progress: longint read FCurValue write SetProgress;
    property ShowText: boolean read FShowText write SetShowText default True;
    property Visible;
    property Font;
    property ShowHint;
    property Align;
    property Anchors;
    property Color;
    property Constraints;
    property Enabled;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;

    Property OnClick;
    Property OnDblClick;
    Property OnPaint;
    Property OnResize;
    Property OnChangeBounds;
    Property OnContextPopup;
    Property OnEndDrag;
    Property OnMouseDown;
    Property OnMouseUp;
    Property OnMouseEnter;
    Property OnMouseLeave;
    Property OnMouseMove;
    Property OnMouseWheel;
    Property OnMouseWheelDown;
    Property OnMouseWheelUp;
    Property OnMouseWheelHorz;
    Property OnMouseWheelLeft;
    Property OnMouseWheelRight;
    Property OnDragDrop;
    Property OnDragOver;
    Property OnStartDrag;
    Property OnStartDock;
    Property OnEndDock;
  end;

implementation

constructor TplShapeProgress.Create(TheOwner: TComponent);
begin
  inherited Create(TheOwner);
  ControlStyle := ControlStyle + [csReplicatable];
  FBorderStyle := bsSingle;
  FFillColor := clBlack;
  FBackColor := clWhite;
  FShowText := True;
  FShapeType := stPie;
  Width := 100;
  Height := 100;
  FFillImage := TPicture.Create;
  FMinValue := 0;
  FMaxValue := 100;
  FCurValue := 0;
end;

destructor TplShapeProgress.Destroy;
begin
  FFillImage.Free;
  inherited Destroy;
end;

procedure TplShapeProgress.Paint;
begin
  case ShapeType of
    stText: DrawText;
    stBarHor,
    stBarVer: DrawBar;
    stRoundBarHor,
    stRoundBarVer: DrawroundBar;
    stPie: DrawPie;
    stImage: DrawImage;
  end;

  if ShowText and (ShapeType <> stText) then
    DrawText;

  inherited Paint;
end;

procedure TplShapeProgress.SetFillColor(AValue: TColor);
begin
  if AValue = FFillColor then
    exit;
  FFillColor := AValue;
  Refresh;
end;

procedure TplShapeProgress.SetProgress(AValue: longint);
begin
  if AValue < FMinValue then
    AValue := FMinValue
  else if AValue > FMaxValue then
    AValue := FMaxValue;
  if FCurValue <> AValue then
  begin
    FCurValue := AValue;
    Refresh;
  end;
end;

procedure TplShapeProgress.AddProgress(AValue: longint);
begin
  Progress := FCurValue + AValue;
  Refresh;
end;

function TplShapeProgress.GetPercentDone: longint;
begin
  Result := trunc((FCurValue / FMaxValue) * 100);
end;

procedure TplShapeProgress.SetBackColor(AValue: TColor);
begin
  if AValue = FBackColor then
    exit;

  FBackColor := AValue;
  Refresh;
end;

procedure TplShapeProgress.SetMinValue(AValue: longint);
begin
  if AValue = FMinValue then
    exit;

  FMinValue := AValue;
  Refresh;
end;

procedure TplShapeProgress.SetMaxValue(AValue: longint);
begin
  if AValue = FMaxValue then
    exit;

  FMaxValue := AValue;
  Refresh;
end;

procedure TplShapeProgress.SetShowText(AValue: boolean);
begin
  if AValue = FShowText then
    exit;

  FShowText := AValue;
  Refresh;
end;

procedure TplShapeProgress.SetFillStyle(AValue: TFPBrushStyle);
begin
  if AValue = FFillStyle then
    exit;

  FFillStyle := AValue;
  Refresh;
end;

procedure TplShapeProgress.SetShapeType(const AValue: TplShapeType);
begin
  if AValue = FShapeType then
    exit;

  FShapeType := AValue;
  Refresh;
end;

procedure TplShapeProgress.SetFillImage(AValue: TPicture);
var
  ximg: TPicture;
begin
  if AValue <> FFillImage then
  begin
    ximg := TPicture.Create;
    ximg.Assign(AValue);
    FFillImage.Bitmap.Height := self.Height;
    FFillImage.Bitmap.Width := self.Width;
    FFillImage.Bitmap.Canvas.StretchDraw(Rect(0, 0, Width, Height), ximg.Bitmap);
    ximg.Free;
    Refresh;
  end;
end;

//----------- Draw Functions ------------------------

procedure TplShapeProgress.DrawRoundBar;
var
  FillSize: longint;
  MinSize: longint;
begin

  with Canvas do
  begin
    Brush.Color := BackColor;
    Brush.Style := bsSolid;
    MinSize := Min(self.Width, self.Height);
    Pen.Color := clGray;
    RoundRect(0, 0, self.Width - 1, self.Height - 1, MinSize div 4, MinSize div 4);
    Pen.Color := clSilver;
    RoundRect(1, 1, self.Width - 2, self.Height - 2, MinSize div 4, MinSize div 4);
    Brush.Style := FillStyle;
    Brush.Color := FillColor;

    if percentdone > 0 then
      case ShapeType of
        stRoundBarHor:
        begin
          FillSize := Trunc((self.Width - 2) * (PercentDone / 100));
          RoundRect(Rect(1, 1, FillSize, self.Height - 2), MinSize div 4, MinSize div 4);
        end;

        stRoundBarVer:
        begin
          FillSize := Trunc((self.Height - 3) * (PercentDone / 100));
          RoundRect(Rect(1, Self.Height - 2 - FillSize, Self.Width - 2, Self.Height - 2),
            MinSize div 4, MinSize div 4);
        end;
      end;
  end;
end;

procedure TplShapeProgress.DrawBar;
var
  xfs: longint;
begin

  with Canvas do
  begin
    Brush.Color := BackColor;
    Brush.Style := bsSolid;
    Pen.Color := clGray;
    Rectangle(0, 0, self.Width, self.Height);
    Pen.Color := clSilver;
    Rectangle(1, 1, self.Width - 1, self.Height - 1);
    Brush.Style := FillStyle;
    Brush.Color := FillColor;

    if percentdone > 0 then
      case ShapeType of
        stBarHor:
        begin
          xfs := Trunc((self.Width - 4) * (PercentDone / 100));
          FillRect(Rect(2, 2, xfs + 2, self.Height - 2));
        end;
        stBarVer:
        begin
          xfs := Trunc((self.Height - 4) * (PercentDone / 100));
          FillRect(Rect(2, Self.Height - 2 - xfs, Self.Width - 2, Self.Height - 2));
        end;
      end;
  end;
end;

procedure TplShapeProgress.DrawText;
var
  X, Y: integer;
  S: string;
begin
  with Canvas do
  begin

    if ShapeType = stText then
    begin
      Brush.Color := BackColor;
      Brush.Style := bsSolid;
      Pen.Color := clGray;
      Rectangle(0, 0, self.Width, self.Height);
      Pen.Color := clSilver;
      Rectangle(1, 1, self.Width - 1, self.Height - 1);
    end;

    Font := Self.Font;

    S := format('%d%%', [PercentDone]);
    Y := self.Height div 2 - TextHeight(S) div 2;
    X := self.Width div 2 - TextWidth(S) div 2;

    TextRect(self.ClientRect, X, Y, S);
  end;
end;

procedure TplShapeProgress.DrawPie;
var
  MiddleX, MiddleY: integer;
  Angle: double;
begin
  with Canvas do
  begin
    Brush.Color := BackColor;
    Brush.Style := bsSolid;
    Pen.Color := clGray;
    Ellipse(0, 0, self.Width - 1, self.Height - 1);
    Pen.Color := clSilver;
    Ellipse(1, 1, self.Width - 2, self.Height - 2);
    Brush.Style := FillStyle;
    Brush.Color := FillColor;
    if PercentDone > 0 then
    begin
      MiddleX := (self.Width - 2) div 2;
      MiddleY := (self.Height - 2) div 2;
      Angle := (Pi * ((PercentDone / 50) + 0.5));
      Pie(1, 1, self.Width - 2, self.Height - 2,
        integer(Round(MiddleX * (1 - Cos(Angle)))),
        integer(Round(MiddleY * (1 - Sin(Angle)))), MiddleX + 1, 1);
    end;
  end;
end;

procedure TplShapeProgress.DrawImage;
var
  ximg: TPicture;
  xfs, xms: longint;
  SRect, DRect: TRect;
begin
  with Canvas do
  begin
    ximg := TPicture.Create;

    ximg.Bitmap.Width := self.Width;
    ximg.Bitmap.Height := Self.Height;
    ximg.Bitmap.Canvas.StretchDraw(rect(0, 0, self.Width, self.Height), FFillImage.Bitmap);
    Brush.Style := bsclear;
    xms := Min(self.Width + 2, self.Height + 2);
    xfs := Trunc((self.Width - 4) * (PercentDone / 100));
    DRect := Rect(2, 2, xfs + 2, self.Height + 2);
    CopyMode := cmSrcCopy;
    CopyRect(DRect, ximg.Bitmap.Canvas, DRect);
    Pen.Color := clGray;
    RoundRect(0, 0, self.Width - 1, self.Height - 1, xms div 4, xms div 4);
    Pen.Color := clSilver;
    RoundRect(1, 1, self.Width - 2, self.Height - 2, xms div 4, xms div 4);

    ximg.Free;
  end;
end;

end.

